#include "FTFP_BERT.hh"
#include "G4RunManager.hh"
#include "G4UImanager.hh"
// #include "QGSP_BIC_HP.hh"
// #include "QGSP_BERT.hh"
// #include "QBBC.hh"
#include "G4EmParameters.hh"
#include "G4HadronicProcessStore.hh"
#include "G4StepLimiterPhysics.hh"

#include "G4GDMLParser.hh"
#include "G4UIExecutive.hh"
#include "G4VUserDetectorConstruction.hh"
#include "G4VisExecutive.hh"

#include "Randomize.hh"

#include <random>

class DetectorConstruction : public G4VUserDetectorConstruction {
public:
  DetectorConstruction(G4VPhysicalVolume *world) : _world(world) {}
  virtual G4VPhysicalVolume *Construct() { return this->_world; }
  G4VPhysicalVolume *getWorld() const { return this->_world; }

private:
  G4VPhysicalVolume *_world;
};

void VoxelGeometry(G4LogicalVolume *log) {
  static G4RunManager *rm = G4RunManager::GetRunManager();
  rm->ReOptimize(log);
  for (int i = 0; i < log->GetNoDaughters(); ++i) {
    VoxelGeometry(log->GetDaughter(i)->GetLogicalVolume());
  }
}

int main(int argc, char **argv) {
  // Detect interactive mode (if no arguments) and define UI session
  G4UIExecutive *ui = new G4UIExecutive(argc, argv);

  G4GDMLParser parser;
  parser.Read("../MW2X1_run000.gdml");

  DetectorConstruction *detectorConstruction = new DetectorConstruction(parser.GetWorldVolume());
  G4RunManager *runManager = new G4RunManager();
  runManager->SetUserInitialization(detectorConstruction);

  G4VModularPhysicsList *physicsList = new FTFP_BERT();
  runManager->SetUserInitialization(physicsList);

  runManager->Initialize();

  VoxelGeometry(parser.GetWorldVolume()->GetLogicalVolume());

  // Initialize visualization
  G4VisManager *visManager = new G4VisExecutive;
  visManager->Initialize();

  G4UImanager *UImanager = G4UImanager::GetUIpointer();

  // interactive mode
  UImanager->ApplyCommand("/control/execute ../vis.mac");
  ui->SessionStart();
  delete ui;

  // Job termination
  // Free the store: user actions, physics_list and detector_description are
  // owned and deleted by the run manager, so they should not be deleted
  // in the main() program !

  delete visManager;
  delete runManager;
}
